//----------------------------
//
//	JaCoB Script : 	MakeCCP
//					Make Conceptual Cardiovascular System
//
//	copyright: 	Darko Lombardo
//
//
//	date:	2.3.2001
//
//----------------------------

// Window Size (Resolution 1024*768)
// x=920
// y=700

// Center
x0 = 350;
y0 = 370;

//----------------------------
// Cardiovascular System
//----------------------------

rSys = 295; 	// inner radius of SystemicVascularSystem
dSys = 5; 		// diameter of SystemicVascularSystem
rPul = 175; 	// inner radius of PulmonaryVascularSystem
dPul = 5; 		// diameter of PulmonaryVascularSystem
rLV = 30; 		// radius of left ventricle (LV)
rRV = 20; 		// radius of right ventricle (RV)
aLAV=30*Math.PI/180; // angle of left atrium valve
aLVV=50*Math.PI/180; // angle of left ventricle valve
aRAV=130*Math.PI/180; // angle of right atrium valve
aRVV=150*Math.PI/180; // angle of right ventricle valve
aSysWKE=50*Math.PI/180; // angle of Windkessel effect element - dilatation of Aorta
aPulWKE=6*Math.PI/6; // angle of Windkessel effect element - dilatation of pulmonary arteries
aSysR0=-Math.PI/6; // angle of Systemic resistance START
aSysR1=7*Math.PI/6; // angle of Systemic resistance STOP
aPulR0=7*Math.PI/6; // angle of Pulmonary resistance START
aPulR1=-Math.PI/6; // angle of Pulmonary resistance STOP
aol = 0.1*Math.PI/180;	// angle of overlapping of arc elements
N = 50; 		// number of blood elements
Q = 1.0; 		//
aStep = Math.PI/30; // step of angle for drawing arcs
x = new Array();
y = new Array();



// Systemic vascular system
//----------------------------

// Left Ventricle Valve Area - before velocity gauge
angle0 = Math.PI/2 - rRV/rSys;
x[0] = x0 + rSys * Math.cos(angle0);
y[0] = y0 - rSys * Math.sin(angle0);
angle=angle0;
for (i=1; angle > aLVV; i++) {
	x[i] = x0 + (rSys+dSys) * Math.cos(angle);
	y[i] = y0 - (rSys+dSys) * Math.sin(angle);
	angle -= aStep;
}
angle = aLVV;
x[i] = x0 + (rSys+dSys) * Math.cos(angle);
y[i] = y0 - (rSys+dSys) * Math.sin(angle);
i++;
for ( ; angle < angle0; i++) {
	x[i] = x0 + rSys * Math.cos(angle);
	y[i] = y0 - rSys * Math.sin(angle);
	angle += aStep;
}
SysLVV1 = jacob.createPolygonElement("SysLVV1", x, y, i);
SysLVV1.setClosed(false);

// Left Ventricle Valve Area - velocity gauge
Size=5;
angle0 = aLVV+aol;
x[0] = x0 + rSys * Math.cos(angle0);
y[0] = y0 - rSys * Math.sin(angle0);
angle = angle0;
for (i=1; angle > aLVV-Size/rSys; i++) {
	x[i] = x0 + (rSys+dSys) * Math.cos(angle);
	y[i] = y0 - (rSys+dSys) * Math.sin(angle);
	angle -= aStep;
}
angle = aLVV-Size/rSys;
x[i] = x0 + (rSys+dSys) * Math.cos(angle);
y[i] = y0 - (rSys+dSys) * Math.sin(angle);
i++;
for ( ; angle < angle0; i++) {
	x[i] = x0 + rSys * Math.cos(angle);
	y[i] = y0 - rSys * Math.sin(angle);
	angle += aStep;
}
SysVG = jacob.createPolygonElement("SysVG", x, y, i);
SysVG.setClosed(false);

// Left Ventricle Valve Area - after velocity gauge
angle0 = aLVV-Size/rSys+aol;
x[0] = x0 + rSys * Math.cos(angle0);
y[0] = y0 - rSys * Math.sin(angle0);
angle = angle0;
for (i=1; angle > aSysWKE; i++) {
	x[i] = x0 + (rSys+dSys) * Math.cos(angle);
	y[i] = y0 - (rSys+dSys) * Math.sin(angle);
	angle -= aStep;
}
angle = aSysWKE;
x[i] = x0 + (rSys+dSys) * Math.cos(angle);
y[i] = y0 - (rSys+dSys) * Math.sin(angle);
i++;
for ( ; angle < angle0; i++) {
	x[i] = x0 + rSys * Math.cos(angle);
	y[i] = y0 - rSys * Math.sin(angle);
	angle += aStep;
}
SysLVV2 = jacob.createPolygonElement("SysLVV2", x, y, i);
SysLVV2.setClosed(false);

// WindKessel Effect Element Area
angle0 = aSysWKE+aol;
x[0] = x0 + rSys * Math.cos(angle0);
y[0] = y0 - rSys * Math.sin(angle0);
angle = angle0;
for (i=1; angle > aSysR0; i++) {
	x[i] = x0 + (rSys+dSys) * Math.cos(angle);
	y[i] = y0 - (rSys+dSys) * Math.sin(angle);
	angle -= aStep;
}
angle = aSysR0;
x[i] = x0 + (rSys+dSys) * Math.cos(angle);
y[i] = y0 - (rSys+dSys) * Math.sin(angle);
i++;
for ( ; angle < angle0; i++) {
	x[i] = x0 + rSys * Math.cos(angle);
	y[i] = y0 - rSys * Math.sin(angle);
	angle += aStep;
}
SysWKE = jacob.createPolygonElement("SysWKE", x, y, i);
SysWKE.setClosed(false);

// Systemic Resistance Area
angle0 = aSysR0+aol;
x[0] = x0 + rSys * Math.cos(angle0);
y[0] = y0 - rSys * Math.sin(angle0);
angle = angle0;
for (i=1; angle > aSysR1-2*Math.PI; i++) {
	x[i] = x0 + (rSys+dSys) * Math.cos(angle);
	y[i] = y0 - (rSys+dSys) * Math.sin(angle);
	angle -= aStep;
}
angle = aSysR1-2*Math.PI;
x[i] = x0 + (rSys+dSys) * Math.cos(angle);
y[i] = y0 - (rSys+dSys) * Math.sin(angle);
i++;
for ( ; angle < angle0; i++) {
	x[i] = x0 + rSys * Math.cos(angle);
	y[i] = y0 - rSys * Math.sin(angle);
	angle += aStep;
}
SysR = jacob.createPolygonElement("SysR", x, y, i);
SysR.setClosed(false);

// Systemic Venae Area
angle0 = aSysR1+aol;
x[0] = x0 + rSys * Math.cos(angle0);
y[0] = y0 - rSys * Math.sin(angle0);
angle = angle0;
for (i=1; angle > aRAV; i++) {
	x[i] = x0 + (rSys+dSys) * Math.cos(angle);
	y[i] = y0 - (rSys+dSys) * Math.sin(angle);
	angle -= aStep;
}
angle = aRAV;
x[i] = x0 + (rSys+dSys) * Math.cos(angle);
y[i] = y0 - (rSys+dSys) * Math.sin(angle);
i++;
for ( ; angle < angle0; i++) {
	x[i] = x0 + rSys * Math.cos(angle);
	y[i] = y0 - rSys * Math.sin(angle);
	angle += aStep;
}
SysVenae = jacob.createPolygonElement("SysVenae", x, y, i);
SysVenae.setClosed(false);

// Right Atrium Valve Area
angle0 = aRAV+aol;
x[0] = x0 + rSys * Math.cos(angle0);
y[0] = y0 - rSys * Math.sin(angle0);
angle=angle0;
for (i=1; angle > Math.PI/2 + rLV/rSys; i++) {
	x[i] = x0 + (rSys+dSys) * Math.cos(angle);
	y[i] = y0 - (rSys+dSys) * Math.sin(angle);
	angle -= aStep;
}
angle = Math.PI/2 + rLV/rSys;
x[i] = x0 + (rSys+dSys) * Math.cos(angle);
y[i] = y0 - (rSys+dSys) * Math.sin(angle);
i++;
for ( ; angle < angle0; i++) {
	x[i] = x0 + rSys * Math.cos(angle);
	y[i] = y0 - rSys * Math.sin(angle);
	angle += aStep;
}
SysRAV = jacob.createPolygonElement("SysRAV", x, y, i);
SysRAV.setClosed(false);


// Pulmonary vascular system
//----------------------------

y0Pul = y0 - rSys + rPul + dPul + 1; // center of Pulmonary VS - constant represents distance PulVS from SysVS

// Right Ventricle Valve Area - before velocity gauge
angle0 = Math.PI/2 + rLV/rPul;
x[0] = x0 + rPul * Math.cos(angle0);
y[0] = y0Pul - rPul * Math.sin(angle0);
angle=angle0;
for (i=1; angle < aRVV; i++) {
	x[i] = x0 + (rPul+dPul) * Math.cos(angle);
	y[i] = y0Pul - (rPul+dPul) * Math.sin(angle);
	angle += aStep;
}
angle = aRVV;
x[i] = x0 + (rPul+dPul) * Math.cos(angle);
y[i] = y0Pul - (rPul+dPul) * Math.sin(angle);
i++;
for ( ; angle > angle0; i++) {
	x[i] = x0 + rPul * Math.cos(angle);
	y[i] = y0Pul - rPul * Math.sin(angle);
	angle -= aStep;
}
PulRVV1 = jacob.createPolygonElement("PulRVV1", x, y, i);
PulRVV1.setClosed(false);

// Right Ventricle Valve Area - velocity gauge
Size=5;
angle0 = aRVV-aol;
x[0] = x0 + rPul * Math.cos(angle0);
y[0] = y0Pul - rPul * Math.sin(angle0);
angle=angle0;
for (i=1; angle < aRVV+Size/rPul; i++) {
	x[i] = x0 + (rPul+dPul) * Math.cos(angle);
	y[i] = y0Pul - (rPul+dPul) * Math.sin(angle);
	angle += aStep;
}
angle = aRVV+Size/rPul;
x[i] = x0 + (rPul+dPul) * Math.cos(angle);
y[i] = y0Pul - (rPul+dPul) * Math.sin(angle);
i++;
for ( ; angle > angle0; i++) {
	x[i] = x0 + rPul * Math.cos(angle);
	y[i] = y0Pul - rPul * Math.sin(angle);
	angle -= aStep;
}
PulVG = jacob.createPolygonElement("PulVG", x, y, i);
PulVG.setClosed(false);

// Right Ventricle Valve Area - after velocity gauge
angle0 = aRVV+Size/rPul-aol;
x[0] = x0 + rPul * Math.cos(angle0);
y[0] = y0Pul - rPul * Math.sin(angle0);
angle=angle0;
for (i=1; angle < aPulWKE; i++) {
	x[i] = x0 + (rPul+dPul) * Math.cos(angle);
	y[i] = y0Pul - (rPul+dPul) * Math.sin(angle);
	angle += aStep;
}
angle = aPulWKE;
x[i] = x0 + (rPul+dPul) * Math.cos(angle);
y[i] = y0Pul - (rPul+dPul) * Math.sin(angle);
i++;
for ( ; angle > angle0; i++) {
	x[i] = x0 + rPul * Math.cos(angle);
	y[i] = y0Pul - rPul * Math.sin(angle);
	angle -= aStep;
}
PulRVV2 = jacob.createPolygonElement("PulRVV2", x, y, i);
PulRVV2.setClosed(false);

// 'WindKessel' Effect Element Area - dilatation of pulmonary arteries
angle0 = aPulWKE-aol;
x[0] = x0 + rPul * Math.cos(angle0);
y[0] = y0Pul - rPul * Math.sin(angle0);
angle=angle0;
for (i=1; angle < aPulR0; i++) {
	x[i] = x0 + (rPul+dPul) * Math.cos(angle);
	y[i] = y0Pul - (rPul+dPul) * Math.sin(angle);
	angle += aStep;
}
angle = aPulR0;
x[i] = x0 + (rPul+dPul) * Math.cos(angle);
y[i] = y0Pul - (rPul+dPul) * Math.sin(angle);
i++;
for ( ; angle > angle0; i++) {
	x[i] = x0 + rPul * Math.cos(angle);
	y[i] = y0Pul - rPul * Math.sin(angle);
	angle -= aStep;
}
PulWKE = jacob.createPolygonElement("PulWKE", x, y, i);
PulWKE.setClosed(false);

// Pulmonary Resistance Area
angle0 = aPulR0 - 2*Math.PI - aol;
x[0] = x0 + rPul * Math.cos(angle0);
y[0] = y0Pul - rPul * Math.sin(angle0);
angle=angle0;
for (i=1; angle < aPulR1; i++) {
	x[i] = x0 + (rPul+dPul) * Math.cos(angle);
	y[i] = y0Pul - (rPul+dPul) * Math.sin(angle);
	angle += aStep;
}
angle = aPulR1;
x[i] = x0 + (rPul+dPul) * Math.cos(angle);
y[i] = y0Pul - (rPul+dPul) * Math.sin(angle);
i++;
for ( ; angle > angle0; i++) {
	x[i] = x0 + rPul * Math.cos(angle);
	y[i] = y0Pul - rPul * Math.sin(angle);
	angle -= aStep;
}
PulR = jacob.createPolygonElement("PulR", x, y, i);
PulR.setClosed(false);

// Pulmonary Venae Area
angle0 = aPulR1 - aol;
x[0] = x0 + rPul * Math.cos(angle0);
y[0] = y0Pul - rPul * Math.sin(angle0);
angle=angle0;
for (i=1; angle < aLAV; i++) {
	x[i] = x0 + (rPul+dPul) * Math.cos(angle);
	y[i] = y0Pul - (rPul+dPul) * Math.sin(angle);
	angle += aStep;
}
angle = aLAV;
x[i] = x0 + (rPul+dPul) * Math.cos(angle);
y[i] = y0Pul - (rPul+dPul) * Math.sin(angle);
i++;
for ( ; angle > angle0; i++) {
	x[i] = x0 + rPul * Math.cos(angle);
	y[i] = y0Pul - rPul * Math.sin(angle);
	angle -= aStep;
}
PulVenae = jacob.createPolygonElement("PulVenae", x, y, i);
PulVenae.setClosed(false);

// Left Atrium Valve Area
angle0 = aLAV -aol;
x[0] = x0 + rPul * Math.cos(angle0);
y[0] = y0Pul - rPul * Math.sin(angle0);
angle=angle0;
for (i=1; angle < Math.PI/2 - rRV/rPul; i++) {
	x[i] = x0 + (rPul+dPul) * Math.cos(angle);
	y[i] = y0Pul - (rPul+dPul) * Math.sin(angle);
	angle += aStep;
}
angle = Math.PI/2 - rRV/rPul;
x[i] = x0 + (rPul+dPul) * Math.cos(angle);
y[i] = y0Pul - (rPul+dPul) * Math.sin(angle);
i++;
for ( ; angle > angle0; i++) {
	x[i] = x0 + rPul * Math.cos(angle);
	y[i] = y0Pul - rPul * Math.sin(angle);
	angle -= aStep;
}
PulLAV = jacob.createPolygonElement("PulLAV", x, y, i);
PulLAV.setClosed(false);



// Ventricles
//---------------------------

LV = jacob.createCircleElement("LeftVentricle", x0+rRV+1, y0-rSys+dSys, rLV);
LV.setClosed(false);
RV = jacob.createCircleElement("RightVentricle", x0-rLV-1, y0-rSys+dSys, rRV);
RV.setClosed(false);



// Systemic Vascular Resistance
//---------------------------

dRSys = 4; 	// distance between Sys.Vasc.Resistance Element and Sys.Vasc.System
Size = 100;
N = 5;

x[0] = x0-Size/2;
y[0] = y0+rSys+dSys+dRSys;
RSys = jacob.createRectangleElement("SystemicVascularResistance", x[0], y[0], Size, 4);

for (i=Size/(N+1); i<Size; i+=Size/(N+1)) {
	particle = jacob.createParticle("SystemicVascularResistance", -Q, x[0]+i, y[0]+2);
	particle.setFixed(true);
}


// Pulmonary Vascular Resistance
//---------------------------

dRPul = 4; 	// distance between Pul.Vasc.Res.Element and Vascular System
Size = 50;
N = 3;

x[0] = x0-Size/2;
y[0] = y0 - rSys + 2*rPul + 2*dPul + 1 + dRPul;
RPul = jacob.createRectangleElement("PulmonaryVascularResistance", x[0], y[0], Size, 4);

for (i=Size/(N+1); i<Size; i+=Size/(N+1)) {
	particle = jacob.createParticle("PulmonaryVascularResistance", -Q, x[0]+i, y[0]+2);
	particle.setFixed(true);
}

//
// VALVES
//
//---------------------------

// Left Ventricle Valve (LVV)
//---------------------------

Size = 3*dSys;

x[0] = x0 + (rSys+Size) * Math.cos(aLVV+2/rSys);
y[0] = y0 - (rSys+Size) * Math.sin(aLVV+2/rSys);
x[1] = x0 + (rSys) * Math.cos(aLVV+2/rSys);
y[1] = y0 - (rSys) * Math.sin(aLVV+2/rSys);
x[2] = x0 + (rSys) * Math.cos(aLVV-2/rSys);
y[2] = y0 - (rSys) * Math.sin(aLVV-2/rSys);
x[3] = x0 + (rSys+Size) * Math.cos(aLVV-2/rSys);
y[3] = y0 - (rSys+Size) * Math.sin(aLVV-2/rSys);

jacob.createPolygonElement("LVV", x, y, 4);
particle = jacob.createParticle("LVV", Q, x0 + (rSys+dSys/2) * Math.cos(aLVV), y0 - (rSys+dSys/2) * Math.sin(aLVV));
particle.setFixed(true);
// rotate LVV in degrees
jacob.rotate("LVV", 30);
// set Rotator modifier
angle = aLVV+30/(rSys+Size);
x[0] = x0 + (rSys+Size) * Math.cos(angle);
y[0] = y0 - (rSys+Size) * Math.sin(angle);
jacob.setVectorModifier("LVV","Rotator", x[0], y[0], true);

// spring
angle = aLVV-6/rSys;
x[0] = x0 + (rSys+1.5*Size) * Math.cos(angle);
y[0] = y0 - (rSys+1.5*Size) * Math.sin(angle);
jacob.createCircleElement("LVVspring", x[0], y[0], 2);
particle = jacob.createParticle("LVVspring", Q, x[0], y[0]);
particle.setFixed(true);


// Left Atrium Valve (LAV)
//---------------------------

Size = 3*dPul;
angle = aLAV+90/rPul;

x[0] = x0 + (rPul+dPul) * Math.cos(angle+2/rPul);
y[0] = y0Pul - (rPul+dPul) * Math.sin(angle+2/rPul);
x[1] = x0 + (rPul+dPul-Size) * Math.cos(angle+2/rPul);
y[1] = y0Pul - (rPul+dPul-Size) * Math.sin(angle+2/rPul);
x[2] = x0 + (rPul+dPul-Size) * Math.cos(angle-2/rPul);
y[2] = y0Pul - (rPul+dPul-Size) * Math.sin(angle-2/rPul);
x[3] = x0 + (rPul+dPul) * Math.cos(angle-2/rPul);
y[3] = y0Pul - (rPul+dPul) * Math.sin(angle-2/rPul);

jacob.createPolygonElement("LAV", x, y, 4);
particle = jacob.createParticle("LAV", Q, x0 + (rPul+dPul/2) * Math.cos(angle), y0Pul - (rPul+dPul/2) * Math.sin(angle));
particle.setFixed(true);
// rotate LAV in degrees
jacob.rotate("LAV", 30);

// spring
angle += 6/rPul;
x[0] = x0 + (rPul-1.5*Size) * Math.cos(angle);
y[0] = y0Pul - (rPul-1.5*Size) * Math.sin(angle);
jacob.createCircleElement("LAVspring", x[0], y[0], 2);
particle = jacob.createParticle("LAVspring", Q, x[0], y[0]);
particle.setFixed(true);

// set Rotator modifier
angle = aLAV+60/rPul;
x[0] = x0 + (rPul+dPul-Size) * Math.cos(angle);
y[0] = y0Pul - (rPul+dPul-Size) * Math.sin(angle);
jacob.setVectorModifier("LAV","Rotator", x[0], y[0], true);




// Right Ventricle Valve (RVV)
//---------------------------



// Right Atrium Valve (RAV)
//---------------------------



// WindKessel Effect Element - Systemic
//---------------------------



// WindKessel Effect Element - dilatation of pilmonary arteries
//---------------------------





/*
// Blood particles
//---------------------------

for (i=0; i < 2*Math.PI; i += 2*Math.PI/N) {
	x = x0 + (rVascular + dVascular/2)*Math.cos(i);
	y = y0 - (rVascular + dVascular/2)*Math.sin(i);
	jacob.createParticle("VascularSystem", Q, x, y);
}
*/